/*Robotics 3 - Robot Car
   Bluetooth control using HC-05 BT module
   Android app:  Arduino Bluetooth Controlled Joystick, Unica Robotics
   https://play.google.com/store/apps/details?id=uncia.robotics.joystick&hl=en_US&gl=US

   1.  Parse incoming data string to Angle, Magnitude, and Button state
   2.  Convert string data to integer data
   3.  Convert polar coordinates to rectangular coordinates
*/

#include "SoftwareSerial.h"
#define AREF 3.32;
#define FREQUENCY 3
#define DUTY_CYCLE 0.2

#define PERIOD 1000/FREQUENCY
#define ON_TIME DUTY_CYCLE * PERIOD
#define OFF_TIME PERIOD - ON_TIME
#include "FastLED.h"
CRGB leds[1];
int analoginput = 1; // our analog pin
int analogamount = 0; // stores incoming value
float percentage = 0; // used to store our percentage value
float voltage = 0; // used to store voltage value

// Variables will change:
int ledState = LOW;             // ledState used to set the LED
int dState = 1; //1 means straight, 2 means reverse, 3 means turn

// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long timer;
unsigned long previousMillis = 0;        // will store last time LED was updated
unsigned long uMillis = 0;
unsigned long dMillis = 0;
unsigned long tMillis = 0;
// constants won't change:
const long interval = 500;           // interval at which to blink (milliseconds)
const long uInterval = 50;
const long dInterval = 1500;


//set motor speed, left motor should be slightly faster than right motor


// Left motor pin integers are defined
int enA = 6;
int in1 = 5;
int in2 = 7;

// Right Motor pin integers are defined
int enB = 11;
int in3 = 8;
int in4 = 9;
//define ultrasonic pins
const int trigPin = 17;
const int echoPin = 18;
long duration;
int distance;
SoftwareSerial MyBluetooth(4, 12); // RX | TX
float x_coord, y_coord;
float rad;
void setup() {
  analogReference(EXTERNAL); // use AREF for reference voltage, measured voltage was 3.32
  FastLED.addLeds<NEOPIXEL, 16>(leds, 1);
  Serial.begin(115200);
  MyBluetooth.begin(38400);  //Baud Rate for BT control mode (default for HC-05 module)
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(echoPin, INPUT); // Sets the echoPin as an Input
  //define pin outputs for both motors
  pinMode(enA, OUTPUT);
  pinMode(enB, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);
  delay(1000);
}
void loop() {
  if (MyBluetooth.available() > 0)
  {
    String value = MyBluetooth.readStringUntil('#');
    if (value.length() == 7)
    {
      //serial code to check magnitude, angle and x & y coordinates
      String angle = value.substring(0, 3);
      String magnitude = value.substring(3, 6);
      String button = value.substring(6, 8);
      Serial.print("angle: "); Serial.print(angle); Serial.print('\t');
      Serial.print("magnitude: "); Serial.print(magnitude); Serial.print('\t');
      Serial.print("button: "); Serial.print(button); Serial.print('\t');

      x_coord = magnitude.toInt() * cos((angle.toInt() * PI) / 180);
      y_coord = magnitude.toInt() * sin((angle.toInt() * PI) / 180);
      int X = round(x_coord);
      int Y = round(y_coord);
      float a = angle.toInt();
      float m = magnitude.toInt();
      float xCoord = map(X, -100, 100, 0, 255);
      float yCoord = map(Y, -100, 100, 0, 255);



      Serial.print("X: "); Serial.print(X); Serial.print('\t');
      Serial.print("Y: "); Serial.print(Y); Serial.println();

      Serial.flush();
      value = "";
      if (a > 0 && a < 90) {//quadrant 1
        float rSpeed = (m / 100) * 100;
        float rMotorSpeed = map(rSpeed, 0, 100, 0, 255);
        float lSpeed = rSpeed - ((90 - a) / 90) * rSpeed;
        float lMotorSpeed = map(lSpeed, 0, 100, 0, 255);
        digitalWrite(in1, LOW);
        digitalWrite(in2, HIGH);
        digitalWrite(in3, LOW);
        digitalWrite(in4, HIGH);
        if (a < 20) {//if angle is low enough do a pivot turn
          digitalWrite(in1, HIGH);
          digitalWrite(in2, LOW);
          lMotorSpeed = 165;
        }
        analogWrite(enA, lMotorSpeed);
        analogWrite(enB, rMotorSpeed);
        Serial.print("lSpeed"); Serial.print(lMotorSpeed);
        Serial.print("rSpeed"); Serial.print(rMotorSpeed);
      }
      if (a > 90 && a < 180) {//quadrant 2
        float lSpeed = (m / 100) * 100;
        float lMotorSpeed = map(lSpeed, 0, 100, 0, 255);
        float rSpeed = lSpeed - ((90 - a) / 90) * lSpeed * -1;
        float rMotorSpeed = map(rSpeed, 0, 100, 0, 255);
        digitalWrite(in1, LOW);
        digitalWrite(in2, HIGH);
        digitalWrite(in3, LOW);
        digitalWrite(in4, HIGH);
        if (a > 160) {//if angle is high enough do a pivot turn
          digitalWrite(in3, HIGH);
          digitalWrite(in4, LOW);
          rMotorSpeed = 185;
        }
        analogWrite(enA, lMotorSpeed);
        analogWrite(enB, rMotorSpeed);
        Serial.print("lSpeed"); Serial.print(lMotorSpeed);
        Serial.print("rSpeed"); Serial.print(rMotorSpeed);

      }
      if (a > 180 && a < 270) {//quadrant 3
        float rSpeed = (m / 100) * 100;
        float rMotorSpeed = map(rSpeed, 0, 100, 0, 255);
        float lSpeed = rSpeed - ((90 - a) / 90) * rSpeed;
        float lMotorSpeed = map(lSpeed, 0, 100, 0, 255);
        digitalWrite(in1, HIGH);
        digitalWrite(in2, LOW);
        digitalWrite(in3, HIGH);
        digitalWrite(in4, LOW);
        analogWrite(enA, lMotorSpeed);
        analogWrite(enB, rMotorSpeed);
        Serial.print("lSpeed"); Serial.print(lMotorSpeed);
        Serial.print("rSpeed"); Serial.print(rMotorSpeed);
       
      }
      if (a > 270 && a < 360) {//quadrant 4
        float lSpeed = (m / 100) * 100;
        float lMotorSpeed = map(lSpeed, 0, 100, 0, 255);
        float rSpeed = lSpeed - ((360 - a) / 90) * lSpeed * -1;
        float rMotorSpeed = map(rSpeed, 0, 100, 0, 255);
        digitalWrite(in1, HIGH);
        digitalWrite(in2, LOW);
        digitalWrite(in3, HIGH);
        digitalWrite(in4, LOW);
        analogWrite(enA, lMotorSpeed);
        analogWrite(enB, rMotorSpeed);
        Serial.print("lSpeed"); Serial.print(lMotorSpeed);
        Serial.print("rSpeed"); Serial.print(rMotorSpeed);
        
      }
      if (m < 20) { //if the joystick is dragged out too little, cut the motors
        analogWrite(enA, 0);
        analogWrite(enB, 0);
      }
      
    }

  }
  checkVoltage();//call battery monitor voltage code function
}
void checkVoltage() { //battery voltage monitor code
  analogamount = analogRead(analoginput);
  percentage = (analogamount / 1024.00);
  voltage = (percentage) * (8.13 / 3.018) * AREF; // in millivolts

  Serial.print("% of AREF: ");
  Serial.print(percentage);
  Serial.print("A1 (mV): ");
  Serial.println(voltage);
  FastLED.setBrightness(50);


  // if the LED is off turn it on and vice-versa:
  if (voltage < 6.6) {
    unsigned long currentMillis = millis();

    if ((ledState == HIGH) && (currentMillis - previousMillis >= ON_TIME)) {
      ledState = LOW;
      previousMillis = currentMillis;
      leds[0] = CRGB::Black; FastLED.show();


    } else if ((ledState == LOW) && (currentMillis - previousMillis >= OFF_TIME)) {
      ledState = HIGH;
      previousMillis = currentMillis;
      leds[0] = CRGB::Red; FastLED.show();


    }
  }

  if (voltage > 7) {
    leds[0] = CRGB::Green; FastLED.show();

  }
  if ((voltage < 7) && (voltage > 6.6)) {
    leds[0] = CRGB::Yellow; FastLED.show();

  }
}
